<?php
defined( 'ABSPATH' ) or exit;   // show a blank page if try to access this file directly

require_once( EWZ_PLUGIN_DIR . 'classes/ewz-base.php' );
require_once( EWZ_PLUGIN_DIR . 'classes/ewz-field.php' );
require_once( EWZ_PLUGIN_DIR . 'classes/ewz-layout.php' );
require_once( EWZ_PLUGIN_DIR . 'classes/validation/ewz-layout-input.php' );
require_once( EWZ_PLUGIN_DIR . 'includes/ewz-common.php' );
require_once( EWZ_CUSTOM_DIR . 'ewz-custom-data.php' );


/**
 * Generate the layout data from $_POST and save it
 *
 * Called from admin.php via ajax, and from ewz_layout_menu below.
 *
 * @return  none
 */
function ewz_check_and_process_layouts()
{
    // check for invalid input and add any missing data not generated by html, like checkboxes
    $clean_input = new Ewz_Layout_Input( $_POST );
    $pdata = $clean_input->get_input_data();

    // set up the 'pg_column' field values
    foreach ( $pdata['forder'] as $col => $value ) {
        $mat = array( );
        preg_match( '/forder_f(\d+)_c(\d+)_/', $value, $mat );
        assert( 3 == count( $mat ) );
        $field = $mat[2];
        $pdata['fields'][$field]['pg_column'] = $col;
    }

    // create empty values for 'restrictions' and 'extra_cols' if they dont exist
    if ( !array_key_exists( 'restrictions', $pdata ) ) {
        $pdata['restrictions'] = array( );
    }
    if ( !array_key_exists( 'extra_cols', $pdata ) ) {
        $pdata['extra_cols'] = array( );
    }

    // create the layout object and save
    $layout = new Ewz_Layout( $pdata );

    $layout->save();
}


/* * ************************** Main Layout Function ********************************
 *
 * Generate the Layouts page
 *
 * If getting post input, save the data.
 * Display the Layouts page
 *
 * @param  None
 * @return None
 */

function ewz_layout_menu()
{
    // check the permissions
    if ( !Ewz_Permission::can_see_layout_page() ) {
        wp_die( "Sorry, you do not have permission to view this page" );
    }

    // grab any error message from processing the POST data and display it in an alert
    $message = '';
    if( isset( $_POST['ewzmode'] ) ) {
        try {
            ewz_check_and_process_layouts( );
        } catch( Exception $e ) {
            $message .= $e->getMessage();
            $message .= "\n";
        }
    }

    // get the data from the database and display it
    // errors encountered here stop the processing
    try {
        /**********************************************************/
        /*   Get all the layouts -- including the attached fields */
        /**********************************************************/
        // get editable layouts sorted by layout_id
        $editable_layouts = array_values( Ewz_Layout::get_all_layouts( 'Ewz_Permission', 'can_edit_layout' ) );
        foreach ( $editable_layouts as $k => $layout ) {
            // add an "forder" ( array ) component to the layout to specify the field order
            // -- saves having to sort by pg_column in javascript
            $forder = array();
            foreach ( $layout->fields as $fid => $f ) {
                $forder[$f->pg_column] = $fid;
            }
            $editable_layouts[$k]->forder = $forder;
        }

        $nonce_string = wp_nonce_field( 'ewzadmin', 'ewznonce', true, false );

        $layouts = ewz_html_esc( $editable_layouts );

        /************************************/
        /* Set the ewzG Javascript variable */
        /************************************/

        $ewzG = array( 'layouts' => $layouts );
        $ewzG['can_do'] = Ewz_Permission::can_edit_all_layouts();

        // get option list of editable layouts sorted by layout_id to match the order of the layout objects
        $ewzG['layouts_options'] = ewz_option_list( ewz_html_esc( Ewz_Layout::get_layout_opt_array( 'Ewz_Permission', 'can_edit_layout' ) ) );
        $ewzG['nonce_string'] = $nonce_string;
        $ewzG['message'] = esc_html( $message );
        $ewzG['load_gif'] = plugins_url( 'images/loading.gif', dirname( __FILE__ ) );
        $ewzG['empty_img'] = array( "max_img_w" => EWZ_DEFAULT_DIM,
                                    "max_img_h" => EWZ_DEFAULT_DIM,
                                    "canrotate" => false,
                                    "max_img_size" => EWZ_MAX_SIZE_MB,
                                    "min_img_area" => EWZ_DEFAULT_MIN_AREA,
                                    "allowed_image_types" => array( "image/jpeg", "image/pjpeg" ) );
        $ewzG['empty_str'] = array( "fieldwidth" => EWZ_MAX_FIELD_WIDTH,
                                    "maxstringchars" => EWZ_MAX_STRING_LEN,
                                    "ss_col_fmt" => "-1" );
        $ewzG['empty_opt'] = array( "value" => "", "label" => "", "maxnum" => 0 );
        $ewzG['helpIcon'] = plugins_url( 'images/help.png', dirname( __FILE__ ) );
        $ewzG['maxUploadMb'] = EWZ_MAX_SIZE_MB;
        $ewzG['maxNumitems'] = EWZ_MAX_NUMITEMS;
        $ewzG['maxTotalMb'] = EWZ_MAX_TOTAL_MB;
        $ewzG['display'] = ewz_html_esc( Ewz_Layout::get_all_display_headers() );
        $ewzG['jsvalid'] = Ewz_Base::validate_using_javascript();     // this is set to false for testing server validation

        // a start on internationalization, not completed
        $ewzG['errmsg'] = array(
            'onlylayout' => 'You cannot delete the only layout.' ,
            'deletehasitems' => 'There are items uploaded to webforms using this layout. " .
                "Please delete them first.' ,
            'deletehaswebforms' => 'This action will also delete all the webforms using this layout.' ,
            'deleteconfirm' => 'Really delete the entire layout ' ,
            'layoutname' => 'Please enter a name for the layout.' ,
            'maxnumitems' => 'Please enter the maximum number of items for the layout.' ,
            'nofields' => 'A layout must have at least one field.' ,
            'colhead' => 'Each field in a layout must have a column header.' ,
            'ident' => 'Each field must have an identifier that starts with a letter and " .
                "consists only of letters, digits, dashes and underscores.' ,
            'maximgw' => 'The max image width should consist of digits only.' ,
            'maximgh' => 'The max image height should consist of digits only.' ,
            'minimgarea' => 'The minimum image area should consist of digits only.' ,
            'nomaximgsz' => 'Each image field must have a max image size.' ,
            'maximgsz' => 'The max image size should consist of digits only, " .
                "representing the number of Megabytes allowed.' ,
            'sysmaxsz' => 'The system limits the size of uploaded files to ' . $ewzG['maxUploadMb'] . 'M or less' ,
            'sysmaxup' => 'There is a maximum total upload of ' . $ewzG['maxTotalMb'] .
                'M allowed by your system.  If a user submits all allowed images at " .
                "maximum allowed size, their upload may fail without any message. Continue anyway?' ,
            'optlabel' => 'Each option in an option list must have a label for the web page.' ,
            'optvalue' => 'Each option in an option list must have a value.' ,
            'option' => 'Options may consist only of letters, digits, dashes and underscores.' ,
            'optioncount' => 'A drop-down selection must contain at least one option' ,
            'restrmsg' => 'A restriction must have a message to show the user.' ,
            'maxnumchar' => 'Each text entry must have a maximum number of characters.' ,
            'imgtypes' => 'Each image file must have a set of allowed image types.' ,
            'all_any' => 'Warning: if all the items in a restriction allow `Any` value, " .
                "the restriction has no effect. It will, however, slow down processing. Go ahead anyway?' ,
            'one_any' => 'Warning: if only one item in a restriction is different from `Any`, " .
                "the restriction has the same effect as removing the forbidden item from the selection list, " .
                "but is much slower. Go ahead anyway?' ,
        );

        // TO DO: own version of wp_localize_script?
        // use of $ewzG1.var = ewzG  is a hack to get around the fact that  wp_localize_script
        // runs html_entity_decode on scalar values.  Our data is already processed where it needs to be,
        // and some of it contains html entities which should not be decoded.

        wp_localize_script( 'ewz-admin-layouts',  'ewzG1',  array( 'gvar'=>$ewzG ) );
                              // script name set above in ewz_layout_scripts

        /* **************************************************************** */
        /* The "ewz_layouts" div is where the actual content goes,          */
        /* generated by javascript from the ewzG object to save bandwidth   */
        /*                                                                  */
        /* Within this is a div containing all the help text.  The help     */
        /* divs actually display as popups when the help icons are clicked  */
        /* **************************************************************** */
?>

    <div class="wrap">
        <div id="ewz_layouts" class="ewz-menu-layouts ewz-menu-group">
            <h2>EntryWizard Layout Management</h2>
            <div class="ewz_inotes">
                <b>Notes:</b>
                All items are required.
                <br />Deletions that ask for a confirmation are saved immediately.
                All other changes are saved when you click "Save Changes".
                <br />Restrictions cannot be created until any changes to fields have been saved.
                Further information may be obtained by clicking the help icons:
                <img alt="" class="ewz_ihelp" src="<?php print $ewzG['helpIcon']; ?>">.

            </div>
        </div>

        <div id="help-text" style="display:none">
            <!-- HELP POPUP image area -->
            <div id="imgarea_help" class="wp-dialog" >
                <p>The purpose of this setting is to disallow very small images when they could be enlarged within the maximum width and height limitations. </p>
                <p>It has <b>no</b> effect if the image has width equal to the maximum width or height equal to the maximum height.</p>
        
            </div>
            <!-- HELP POPUP image dimensions -->
            <div id="imgdim_help" class="wp-dialog" >
                 <p>A user attempting to upload an image with width or height greater than the maxima set here will receive an error message.</p>
                 <p>See also the help items on Maximum image size and Minimum image area</p>                                   
            </div>

            <!-- HELP POPUP image size -->
            <div id="imgsize_help" class="wp-dialog" >
                <p>Your web hosting company also puts three limits on uploaded items:<br>
                    <ol><li>No individual item may be bigger than <?php print $ewzG['maxUploadMb']; ?></li>
                        <li>The total size of the whole upload including all items may not be bigger than
                            <?php print $ewzG['maxTotalMb']; ?></li>
                        <li>The maximum number of items that may be uploaded simultaneously is
                            <?php print $ewzG['maxNumitems']; ?></li>
                    </ol>
                </p>
                <p>For the technically-minded, these limits are set in <i>php.ini</i>, and more
                    details may be obtained from the php documentation website
                    http://php.net/manual/en/ini.core.php
                </p>
            </div>

            <!-- HELP POPUP layout -->
            <div id="layout_help" class="wp-dialog" >
                <p>The "name" appears on the menu of layouts on the "Webforms" page.
                </p>

                <p>The shortcode "ewz_show_webform" generates a webform with N rows, where
                    N is the maximum number of items, and a column for each field
                </p>

                <p>If some values in the "Maximum number of items" dropbox are disabled, it it because
                   you have set a larger maximum in one of your option fields.</p>
                <p>Note that your environment or your web hosting company puts three limits on
                    uploaded items:<br>
                    <ol><li>No individual item may be bigger than <?php print $ewzG['maxUploadMb']; ?></li>

                        <li>The total size of the whole upload including all items may not be bigger than
                            <?php print $ewzG['maxTotalMb']; ?></li>

                        <li>The maximum number of items that may be uploaded simultaneously is
                            <?php print $ewzG['maxNumitems']; ?></li>
                    </ol>
                </p>
                <p>( For the technically-minded, these limits are set in <i>php.ini</i>, and more
                    details may be obtained from the php documentation website
                    http://php.net/manual/en/ini.core.php )
                </p>

                <p>In addition, Wordpress currently is able to handle only the following image types:
                    <ol><li>.jpg</li>
                        <li>.jpeg</li>
                        <li>.png</li>
                        <li>.gif</li>
                    </ol>
                </p>
            </div>

            <!-- HELP POPUP fields -->
            <div id="field_help" class="wp-dialog" >
                <p>The shortcode "ewz_show_webform" generates a webform with a column for each field
                    you create.  Each field may be either a text input, an image upload or a
                    drop-down selection.
                </p>
                <p>The order of the fields in the webform may be changed by dragging the "postboxes".
                    ( To move a postbox to the top you may have to drag the top one down, instead)
                </p>
                <p> ** Too many fields, or fields that are too wide, may make a form that is too wide
                    to display on the viewer's monitor.  In that case, the form will display with a
                    scrollbar at the bottom, which is not very user-friendly.  When designing your
                    layout, keep testing the view with different window widths and font sizes.
                    If possible, view it on different monitors with different resolutions.
                </p>
                <p> If a restriction has been created that forbids a certain combination of field
                    values, the field may no longer be deleted, and its type may no longer be changed.
                    If it is a drop-down field with a restriction on one or more options, those options
                    may not be deleted, and their values may not be changed.
                    Items that may no longer be changed for this reason are outlined in red.
                </p>
            </div>

            <!-- HELP POPUP restrictions -->
            <div id="restr_help" class="wp-dialog" >
                <p>Normally, any combination of allowed field values is allowed.
                    There may be occasions where you wish to disallow some particular combination
                </p>
                <p>For instance, you may have a field <i>Type</i> with values <i>Digital</i>,
                    <i>Print</i> or <i>Slide</i>, and a field for an uploaded image <i>File</i>.
                    In this case, you may need to require that a <i>Digital</i> type must have a
                    <i>File</i>, although it may be optional for <i>Print</i> or <i>Slide</i>
                </p>
                <p>To do this, you may create a restriction forbidding the combination
                    <i>Type=Digital</i> and <i>File=Blank</i>, with the message
                    "A Digital type must have an uploaded file".
                </p>
                <p>If a user clicks "Save Changes" when any item has one of these combinations,
                    your message will pop up and the upload will not work.
                </p>
                <p><br><b>Set up all your fields and click "Save Changes" first before adding
                        any restrictions</b>.  Once a field has had a restriction placed on it,
                        some items within the field may no longer be edited. These fields are
                        indicated by a red outline. If you need to change them, you must first
                        delete the restriction.
                </p>
            </div>

            <!-- HELP POPUP restriction message -->
            <div id="rmsg_help" class="wp-dialog" >
                <p>If a user attempts to submit a webform with the forbidden combination of
                    field values, the upload will fail and this message will be shown.</p>
            </div>

            <!-- HELP POPUP field header -->
            <div id="fhead_help" class="wp-dialog" >
                <p>This will appear as a header for the field column on the page the user sees.</p>
                <p>If you have many fields, you may need to keep the total width of your form in
                    mind when setting this value.</p>
            </div>

            <!-- HELP POPUP field type -->
            <div id="ftype_help" class="wp-dialog" >
                <p>You may select the type of input you are asking for.
                </p>
                <p>Currently, this may be one of:
                    <ol>
                        <li><b>A text input:</b> the user may input any piece of text,
                            subject only to the length restrictions you set</li>
                        <li><b>An image file:</b> the user will be asked to select a file from their
                            local machine.  You may restrict various image parameters, such a size
                            and dimensions.</li>
                        <li><b>A drop-down option list:</b> you choose the values that appear.
                            The user must select one of these values.</li>
                    </ol>
                </p>
            </div>

            <!-- HELP POPUP spreadsheet header -->
            <div id="sshead_help" class="wp-dialog" >
                <p>This identifier is used for the header that appears at the top of the column
                    if this field is included in your spreadsheet.
                </p>
                <p>If the field is an image file, you may also (optionally) choose to include it
                    as part of the filename when you download the image.
                </p>
                <p>It must start with a letter and consist only of letters, digits, dashes or
                    underscores - no spaces or special characters.
                    This makes sure that a filename containing it should be accepted by most
                    systems.
                </p>
            </div>

            <!-- HELP POPUP image type -->
            <div id="imgtype_help" class="wp-dialog" >
                <p>Wordpress currently is able to handle only the following image types:
                    <ol><li>.jpg</li>
                        <li>.jpeg</li>
                        <li>.png</li>
                        <li>.gif</li>
                    </ol>
                </p>
                <p>Note that EntryWizard saves images with filenames derived from the user's
                    original filenames in the following way:
                    <ol><li> All characters except letters, digits, underscores, dashes,
                            and all periods except the last,  are replaced by '_'.
                            This ensures the filename is acceptable on most systems.</li>
                        <li>  If the base filename then ends with a digit, another '_' is appended.
                            This makes extra digits added by Wordpress in the case of duplicate
                            filenames more legible. It also ensures that no filename is totally
                            numeric - Wordpress handles files with completely numeric names
                            differently in some cases.</li>
                    </ol>
                </p>
            </div>

            <!-- HELP POPUP spreadsheet column -->
            <div id="sscol_help" class="wp-dialog" >
                <p><b>If you wish the data in this column to appear in the spreadsheet, specify here
                     the column in which it is to appear. If you select 'None', the item will not appear
                     in the spreadsheet at all.
                </p>
                <p>The system will not allow you to select a column you have already chosen for
                    another field
                </p>
            </div>

            <!-- HELP POPUP required -->
            <div id="req_help" class="wp-dialog" >
                <p>Check this box if you wish to insist that the field is filled out by everyone.
                    Leave it unchecked if the field is optional.
                </p>
            </div>

            <!-- HELP POPUP options -->
            <div id="opt_help" class="wp-dialog" >
                <p>For each option you create, you need to select:
                    <ul><li><b>Label for Web Page:</b> the label that is displayed in the drop-down
                            list the user selects from.<br>
                            If you have many fields, you may need to keep the total width of your
                            form in mind when setting this value.<br>
                            May contain only letters, digits, dashes, underscores</li>
                        <li><b>Value for Spreadsheet:</b> what you see in the corresponding
                            spreadsheet column when the user selects this item.<br>
                            May contain only letters, digits, dashes, underscores<br>
                            (Behind the scenes, this is also the value stored in the database) </li>
                        <li><b>Maximum Number Allowed:</b> If you enter a number N here,
                            no more than N items uploaded by this user may have the matching value.<br>
                            This allows for such rules as "6 images may be submitted, no more than
                            4 may be prints."<br>
                            You cannot set a maximum here that is greater than the "Maximum number
                            of items" set in the "General Information" section. </li>
                    </ul>
                    Use the 'X' button to delete an option.
                </p>
            </div>

            <!-- HELP POPUP allow rotation -->
            <div id="rot_help" class="wp-dialog" >
                <p>If this is checked, and you have set a maximum width of 1280 and a maximum
                    height of 1024, an image with height 1280 and width 1024 will also be accepted.
                </p>
                <p>May be useful if the image is to be printed, or displayed on a rotatable monitor.
                </p>
            </div>

            <!-- HELP POPUP max text chars -->
            <div id="maxchar_help" class="wp-dialog" >
                <p>The user will not be able to type more than this many characters in the text box.
                </p>
            </div>

            <!-- HELP POPUP max visible chars -->
            <div id="maxvis_help" class="wp-dialog" >
                <p>The number of characters that will be visible at any one time - ie the width of
                    the text box.
                </p>
                <p>If you have many fields, you may need to keep the total width of your form in mind
                    when setting this value.
                </p>
                <p>On the other hand, a box that is smaller than the number of characters the user
                    wishes to type is not very comfortable to use.
                </p>
            </div>

            <!-- HELP POPUP reformatted text -->
            <div id="frmtstr_help" class="wp-dialog" >
                <p>If a column is selected, that column in the spreadsheet will contain this item
                    re-formatted so that each word starts with a capital letter.
                </p>
                <p>Useful for titles used in displays.
                </p>
                <p>( The text is shown unchanged if it already contains both upper and
                    lower-case characters )
                </p>
            </div>

            <!-- HELP POPUP extra info -->
            <div id="xtra_help" class="wp-dialog" >
                <p>In addition to the information entered by members, you may include several
                    other items of information in the spreadsheet that you download.
                </p>
                <p>The source of the data is shown in parentheses.
                  <ul><li>"WP User data" is information you control via the Users menu </li>
                      <li>"EWZ Item data" is information about an item stored by this plugin but not
                          actually set by the user</li>
                     <li>"EWZ Webform data" is set by you on the WebForms page, except for the WP ID,
                         which is a numeric identifier created by Wordpress</li>
                     <li>"Admin Uploaded data" is data that you may optionally upload as a .csv file
                         via the WebForms page.  It is blank if you did not upload anything for the
                         item</li>
                     <li>"Custom data" is again optional. Wordpress stores some information about a
                         user, but there are plugins, like CIMI User Extra Fields,  that allow you
                         to add more information.
                         If you have such a plugin, and if it provides a function to access the
                         information, you may tell EntryWizard about it in the "ewz-custom-data.php"
                         file.</li>
                  </ul>
               </p>
            </div>
        </div>
    </div> <!-- wrap -->



    <?php
    } catch( Exception $e ){
            wp_die( $e->getMessage() );
    }
}   // end function ewz_layouts_menu

/************************ End Main Layout Function ********************/
